/*
** Module   : AppServerInvocationResult.java
** Abstract : Container for the result of a remote appserver procedure call. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130529 Created initial version.
** 002 CA  20130829 Return the resolved parameter modes to the caller too.
** 003 CA  20131220 Extracted data common with async web service calls to RemoteInvocationResult.
*/

package com.goldencode.p2j.main;

import java.io.*;

import com.goldencode.p2j.util.*;

/**
 * This class acts as a container for the various data which is returned by or updated during 
 * remote appserver calls.  This includes the function/procedure return value, updated 
 * OUTPUT/INPUT-OUTPUT parameters or the persistent procedure handle for a RUN ... PERSISTENT
 * call.
 */
public class AppServerInvocationResult
extends RemoteInvocationResult
implements Externalizable
{
   /** The remote procedure handle, used to build a {@link ProxyProcedureWrapper} instance. */
   private String procedureId;
   
   /** The connection ID, returned during the connect request. */
   private String connectionID;
   
   /**
    * Serialize this object, to be picked up by the other side.
    *  
    * @param    out
    *           The output destination to which the data will be sent.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeInt(arguments.length);
      for (int i = 0; i < arguments.length; i++)
      {
         out.writeObject(arguments[i]);
      }
      
      out.writeObject(result);
      out.writeObject(procedureId);
      out.writeObject(error);
      out.writeObject(connectionID);
      out.writeObject(modes);
   }

   /**
    * Used the specified input source and initialize this instance. 
    *  
    * @param    in
    *           The input source from which the data will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      int parameterCount = in.readInt();
      arguments = new Object[parameterCount];
      for (int i = 0; i < parameterCount; i++)
      {
         arguments[i] = in.readObject();
      }

      result = (BaseDataType) in.readObject();
      procedureId = (String) in.readObject();
      error = (Throwable) in.readObject();
      connectionID = (String) in.readObject();
      modes = (String) in.readObject();
   }

   /**
    * Get the handle code for the remote persistent procedure.
    * 
    * @return   The {@link #procedureId} field.
    */
   public String getProcedureId()
   {
      return procedureId;
   }

   /**
    * Save the remote persistent procedure handle's code.
    * 
    * @param    procedureId
    *           The code for the remote persistent procedure handle.
    */
   public void setProcedureId(String procedureId)
   {
      this.procedureId = procedureId;
   }

   /**
    * Get the established connection ID, for a connect request.
    * 
    * @return   See above.
    */
   public String getConnectionID()
   {
      return connectionID;
   }

   /**
    * Set the new connection ID, after a connect request.
    * 
    * @param    connectionID
    *           The connection ID, which will be unknown if there are any {@link #error errors}.
    */
   public void setConnectionID(String connectionID)
   {
      this.connectionID = connectionID;
   }
   
   /**
    * Check if the RETURN-VALUE will be set.
    * 
    * @return   In case of appserver results, it returns <code>true</code>.
    */
   public boolean setReturnValue()
   {
      return true;
   }
}
